searchData={"items":[{"type":"module","title":"ExampleTest","doc":"A utility for defining eminently readable example-based tests.\n\nExample tests look like this:\n\n    example_test \"grants free shipping based on the marketing site's stated policy\",\n                 \\\"\\\"\\\"\n                 | spending_by_category          | coupon      | gets_free_shipping? |\n                 | %{shoes: 19_99, pants: 29_99} |             | false               |\n                 | %{shoes: 59_99, pants: 49_99} |             | true                |\n                 | %{socks: 10_99}               |             | true                |\n                 | %{pants: 1_99}                | \"FREE_SHIP\" | true                |\n                 \\\"\\\"\\\",\n                 %{\n                   spending_by_category: spending_by_category,\n                   coupon: coupon,\n                   gets_free_shipping?: gets_free_shipping?\n                 } do\n      shipping_cost = ShippingCalculator.calculate_shipping(spending_by_category, coupon)\n      free_shipping? = shipping_cost == 0\n      assert free_shipping? == gets_free_shipping?\n    end","ref":"ExampleTest.html"},{"type":"module","title":"Why example testing? - ExampleTest","doc":"Example testing reduces toil associated with writing tests that cover\na wide variety of different cases. It also localizes the test logic\ninto a single place, so that at a glance you can see how a number of\ndifferent factors affect the behavior of the system under test.\n\nAs a bonus, a table of examples (with their expected results) often\nmatches how the business communicates the requirements of a system,\nboth internally and to customers—for instance, in a table describing\nshipping costs based on how much a customer spends, where they're\nlocated, whether they've bought a promotional product, etc. This means\nexample tests can often be initially created by pulling directly from\na requirements document that your product folks provided, and the\nproduct folks can later read the tests (or at least the examples table)\nif they want to verify the behavior of the system.","ref":"ExampleTest.html#module-why-example-testing"},{"type":"macro","title":"ExampleTest.example_test/4","doc":"Defines tests that use your example data.\n\nUse it like:\n\n    example_test \"works as expected\", examples, %{value: from_context, expected_result: expected_result} do\n      assert something(from_context) == expected_result\n    end","ref":"ExampleTest.html#example_test/4"},{"type":"function","title":"ExampleTest.parse_examples/2","doc":"","ref":"ExampleTest.html#parse_examples/2"},{"type":"module","title":"ExampleTest.Sigil","doc":"Provides a sigil to wrap parsing of the example test tables.\n\nThe `example_test` macro automatically parses Markdown-style example tables\ninto a list of maps for use in the test contexts, so this sigil is never\n*required* to be used. However, it may occasionally be useful to do things\nlike declare a module attribute which is the pre-parsed example table, or to\ninspect how the table will be parsed.\n\nNote that on Elixir v1.14 and earlier, the sigil is `~X` since those versions\ndon't support multi-character sigils.","ref":"ExampleTest.Sigil.html"},{"type":"function","title":"ExampleTest.Sigil.sigil_EXAMPLES/2","doc":"Provides a sigil for producing example data that you can use in tests.\n\n#","ref":"ExampleTest.Sigil.html#sigil_EXAMPLES/2"},{"type":"function","title":"Examples - ExampleTest.Sigil.sigil_EXAMPLES/2","doc":"You can have an arbitrary number of columns and rows. Headers are parsed\nas atoms, while the individual cells are parsed as Elixir values.\n\n    iex> ~EXAMPLES\"\"\"\n    ...>   | plan      | user_permission | can_invite?     |\n    ...>   | :free     | :admin          | true            |\n    ...>   | :free     | :editor         | \"maybe\"         |\n    ...>   | :free     | :view_only      | false           |\n    ...>   | :standard | :admin          | true            |\n    ...>   | :standard | :editor         | \"tuesdays only\" |\n    ...>   | :standard | :view_only      | false           |\n    ...> \"\"\"\n    [\n      %{plan: :free, user_permission: :admin, can_invite?: true},\n      %{plan: :free, user_permission: :editor, can_invite?: \"maybe\"},\n      %{plan: :free, user_permission: :view_only, can_invite?: false},\n      %{plan: :standard, user_permission: :admin, can_invite?: true},\n      %{plan: :standard, user_permission: :editor, can_invite?: \"tuesdays only\"},\n      %{plan: :standard, user_permission: :view_only, can_invite?: false}\n    ]\n\nYou can optionally include separators between the headers and the data.\n\n    iex> ~EXAMPLES\"\"\"\n    ...>   | plan      | user_permission | can_invite?     |\n    ...>   |-----------|-----------------|-----------------|\n    ...>   | :free     | :admin          | true            |\n    ...>   | :free     | :editor         | \"maybe\"         |\n    ...> \"\"\"\n    [\n      %{plan: :free, user_permission: :admin, can_invite?: true},\n      %{plan: :free, user_permission: :editor, can_invite?: \"maybe\"}\n    ]\n\nYou can pass the output of `~EXAMPLES` directly to the `example_test` macro:\n\n    example_test \"distinguishes even and odd numbers\",\n                  ~EXAMPLES\"\"\"\n                  | even  | odd |\n                  | 2     | 1   |\n                  | 4     | 3   |\n                  | 6     | 5   |\n                  \"\"\",\n                  %{even: even, odd: odd} do\n      assert rem(even, 2) == 0\n      assert rem(odd, 2) == 1\n    end","ref":"ExampleTest.Sigil.html#sigil_EXAMPLES/2-examples"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.0.1 - Changelog","doc":"Initial release.","ref":"changelog.html#v0-0-1"},{"type":"extras","title":"ExampleTest","doc":"# ExampleTest\n\n[![Hex.pm](https://img.shields.io/hexpm/v/example_test)](https://hex.pm/packages/example_test) [![Build and Test](https://github.com/s3cur3/example_test/actions/workflows/elixir-build-and-test.yml/badge.svg)](https://github.com/s3cur3/example_test/actions/workflows/elixir-build-and-test.yml) [![Elixir Quality Checks](https://github.com/s3cur3/example_test/actions/workflows/elixir-quality-checks.yml/badge.svg)](https://github.com/s3cur3/example_test/actions/workflows/elixir-quality-checks.yml) [![Code coverage](https://codecov.io/gh/s3cur3/example_test/graph/badge.svg)](https://codecov.io/gh/s3cur3/example_test)\n\nA utility for defining eminently readable example-based tests in \nElixir's ExUnit, inspired by [example tests in Cucumber](https://cucumber.io/docs/guides/10-minute-tutorial/?lang=java#using-variables-and-examples).","ref":"readme.html"},{"type":"extras","title":"What are example tests? - ExampleTest","doc":"Example tests let you define variables along a number of dimensions \nand re-run the same test body (including all `setup`) for each \ncombination of variables.\n\nAn extremely simple (perhaps too simple!) example:\n\n```elixir\nsetup context do\n  # context.permissions gets set by the example_test below\n  permissions = Map.get(context, :permissions, nil)\n  user = AccountsFixtures.user_fixture{permissions: permissions}\n  %{user: user}\nend\n\nexample_test \"users can view the post regardless of permission level\",\n             \"\"\"\n             | permissions |\n             |-------------|\n             | :admin      |\n             | :editor     |\n             | :viewer     |\n             | nil         |\n             \"\"\",\n             %{user: user, permissions: permissions} do\n  assert Posts.can_view?(user), \"User with #{permissions} permissions should be able to view\"\nend\n```\n\nThat test will run 4 times, with the `:permissions` variable from the table \nbeing applied to the test's context each time (and therefore being made\navailable to the `setup` handler). Thus, under the hood this generates\nfour unique tests, equivalent to doing something like this:\n\n```elixir\nsetup context do\n  permissions = Map.get(context, :permissions, nil)\n  user = AccountsFixtures.user_fixture{permissions: permissions}\n  %{user: user}\nend\n\nfor permission <- [:admin, :editor, :viewer, nil] do\n  @permission permission\n\n  @tag permission: @permission\n  test \"users with #{@permission} can view the post\", %{user: user} do\n    assert Posts.can_view?(user)\n  end\nend\n```\n\nOf course, that example only had a single variable, so you could argue\nthe `for` version is cleaner. But `example_test` supports an arbitrary\nnumber of variables, so you can describe complex business rules like\n\"users get free shipping if they spend more than $100, or if they buy\nsocks, or if they have the right coupon code\":\n\n```elixir\nexample_test \"grants free shipping based on the marketing site's stated policy\",\n             \"\"\"\n             | spending_by_category          | coupon      | gets_free_shipping? |\n             |-------------------------------|-------------|---------------------|\n             | %{shoes: 19_99, pants: 29_99} |             | false               |\n             | %{shoes: 59_99, pants: 49_99} |             | true                |\n             | %{socks: 10_99}               |             | true                |\n             | %{pants: 1_99}                | \"FREE_SHIP\" | true                |\n             \"\"\",\n             %{\n               spending_by_category: spending_by_category,\n               coupon: coupon,\n               gets_free_shipping?: gets_free_shipping?\n             } do\n  shipping_cost = ShippingCalculator.calculate_shipping(spending_by_category, coupon)\n  free_shipping? = shipping_cost == 0\n  assert free_shipping? == gets_free_shipping?\nend\n```","ref":"readme.html#what-are-example-tests"},{"type":"extras","title":"Why example testing? - ExampleTest","doc":"Example testing reduces toil associated with writing tests that cover\na wide variety of different cases. It also localizes the test logic\ninto a single place, so that at a glance you can see how a number of\ndifferent factors affect the behavior of the system under test.\n\nAs a bonus, a table of examples (with their expected results) often\nmatches how the business communicates the requirements of a system,\nboth internally and to customers—for instance, in a table describing\nshipping costs based on how much a customer spends, where they're\nlocated, whether they've bought a promotional product, etc. This means\nexample tests can often be initially created by pulling directly from\na requirements document that your product folks provided, and the\nproduct folks can later read the tests (or at least the examples table)\nif they want to verify the behavior of the system.\n\n#","ref":"readme.html#why-example-testing"},{"type":"extras","title":"Example tests versus property tests - ExampleTest","doc":"Example tests cover a lot of the same ground as property-based testing.\nBoth allow you to write fewer tests while covering more of your system's\nbehavior. This library is not a replacement for property tests, but\nrather complimentary to them.\n\nThere are a few reasons you might choose to write an example\ntest rather than a property test:\n\n- **Ease of writing**: Property tests take a lot of practice to get\n  good at writing. They're often quite time consuming to produce, and\n  even when you think you've adequately described the parameters to\n  the system.\n- **For communication with other stakeholders**: An example test\n  table can be made readable by non-programmers (or non-Elixir \n  programmers), so they can be a good way of showing other people\n  in your organization which behaviors of the system you've verified.\n  Because they can compactly express a lot of test cases, they're\n  much more suitable for this than saying \"go read the title of\n  every line in this file that starts with `test`.\"\n- **For verifying the exact scenarios described by other stakeholders**:\n  Sometimes the edges of a particular behavior may be fuzzy—not just\n  to you, but in the business domain as well. Hammering out hard-and-fast\n  rules may not be necessary or worth it, so property tests that exercise\n  the boundaries would be overkill. In contrast, when your product\n  folks produce a document that describes the behavior of particular\n  scenarios, you can encode that in a table and ensure that for the\n  cases that are well-specified, the system behaves correctly.\n\nWhen would you write a property test instead of an example tests?\n\n- When you want the absolute highest confidence in your code.\n- When the correctness of a piece of code is important enough to merit\n  a large time investment in getting the tests right.\n- When the system's behavior at the edges is well specified.","ref":"readme.html#example-tests-versus-property-tests"},{"type":"extras","title":"Installation and writing your first test - ExampleTest","doc":"1. Add `example_test` to your `mix.exs` dependencies:\n\n    ```elixir\n    def deps do\n      [\n        {:example_test, \"~> 0.0.1\", only: [:test]},\n      ]\n    end\n    ```\n2. Run `$ mix deps.get` to download the package\n3. Write your first example test by adding `import ExampleTest` \n   to the top of your test module, and using the `example_test` macro.\n\n   You can optionally include a separator between the header and body\n   of the table (like `|--------|-------|`).\n\n   The header of your table will be parsed as atoms to pass into your\n   test context. The body cells of the table can be any valid Elixir \n   expression, and empty cells will produce a `nil` value.\n\n   A dummy example:\n\n    ```elixir\n    defmodule MyApp.MyModuleTest do\n      use ExUnit.Case, async: true\n      import ExampleTest\n\n      example_test \"behaves as expected\",\n                   \"\"\"\n                   | variable_1       | variable_2           | etc     |\n                   |------------------|----------------------|---------|\n                   | %{foo: :bar}     | div(19, 3)           | false   | \n                   | \"bip bop\"        | String.upcase(\"foo\") | true    |\n                   | [\"whiz\", \"bang\"] | :ok                  |         |\n                   |                  | nil                  | \"maybe\" |\n                   \"\"\",\n                   %{\n                     variable_1: variable_1,\n                     variable_2: variable_2,\n                     etc: etc\n                   } do\n          assert MyModule.valid_combination?(variable_1, variable_2, etc)\n        end\n    end\n    ```","ref":"readme.html#installation-and-writing-your-first-test"}],"content_type":"text/markdown"}